/*
 *  (C) Copyright 2000 Mulle kybernetiK. All rights reserved. 
 * 
 *  Permission to use, copy, modify and distribute this software and its
 *  documentation is hereby granted, provided that both the copyright
 *  notice and this permission notice appear in all copies of the
 *  software, derivative works or modified versions, and any portions
 *  thereof, and that both notices appear in supporting documentation, 
 *  and that credit is given to Mulle kybernetiK in all documents and
 *  publicity pertaining to direct or indirect use of this code or its
 *  derivatives. 
 * 
 *  THIS IS EXPERIMENTAL SOFTWARE AND IT IS KNOWN TO HAVE BUGS, SOME OF
 *  WHICH MAY HAVE SERIOUS CONSEQUENCES. THE COPYRIGHT HOLDER ALLOWS FREE
 *  USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION. THE COPYRIGHT HOLDER
 *  DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER
 *  RESULTING DIRECTLY OR INDIRECTLY FROM THE USE OF THIS SOFTWARE OR OF
 *  ANY DERIVATIVE WORK.
 *
 *  Coded by Nat!
 *
 *  $Id: MulleSymmetricCipher.m,v 1.1.1.1 2001/02/22 14:55:31 znek Exp $
 *
 *  $Log: MulleSymmetricCipher.m,v $
 *  Revision 1.1.1.1  2001/02/22 14:55:31  znek
 *  Re-import of Nat!'s cryptographic framework. This version has been
 *  ported to MOSX, MOSXS and Solaris. It uses an extended build process
 *  similar to EDCommon & friends.
 *
 *  Revision 1.1.1.1  1970/01/01 22:37:32  nat
 *  Mercyful Release
 *
 */

#import "MulleSymmetricCipher.h"
#import "MulleSymmetricCipherKey.h"


@implementation MulleSymmetricCipher


static NSMutableDictionary  *ciphers;


static inline NSMutableDictionary  *ciphersDictionary()
{
   if( ! ciphers)
      ciphers = [[NSMutableDictionary alloc] init];
   return( ciphers);
}


//
// them bookkeeping methods
//
+ (void) registerCipher:(MulleSymmetricCipher *) cipher
{
   if( ! [cipher isKindOfClass:self])
   {
      fprintf( stderr, "you're trying hard to hose the system, but failing...\n");
      return;
   }
   [ciphersDictionary() setObject:cipher
                           forKey:[cipher name]];
}


+ (void) unregisterCipher:(MulleSymmetricCipher *) cipher
{
   if( ! [cipher isKindOfClass:self])
   {
      fprintf( stderr, "you're trying hard to hose the system, but failing...\n");
      return;
   }

   [ciphersDictionary() removeObjectForKey:[cipher name]];
}

//
// Might look thru plug-in directory, some time in the future...
//
+ (id) cipherWithName:(NSString *) name
{
   return( [ciphersDictionary() objectForKey:name]);
}   

// though, its nowhere written in the documentation, it has always
// been that allValues indices correspond to allKeys indices
// lets be bold and rely on this one more time 
+ (NSArray *) registeredCiphers
{
   return( [ciphersDictionary() allValues]);
}


+ (NSArray *) registeredCipherNames
{
   return( [ciphersDictionary() allKeys]);
}


//
//
//
static void  pain( id self, SEL _cmd)
{
   [NSException raise:NSGenericException
               format:@"You need to implement %@ in subclass %@", NSStringFromSelector( _cmd),
      [self class]];
}


- (Class) keyClass
{
   pain( self, _cmd);
   return( nil);
}



//
// that's the interface you have to override in your subclass
//
- (NSString *) name
{
   pain( self, _cmd);
   return( nil);
}


- (NSData *) encryptedData:(NSData *) data
                   withKey:(MulleSymmetricCipherKey *) key
         withInitialValues:(NSData *) values
{
   pain( self, _cmd);
   return( nil);
}


- (NSData *) decryptedData:(NSData *) data
                   withKey:(MulleSymmetricCipherKey *) key
         withInitialValues:(NSData *) values
{
   pain( self, _cmd);
   return( nil);
}


//
//
//
- (unsigned int) sizeOfInitialValues
{
   return( 0);
}


- (BOOL) mutableDataMethodsAreFast
{
   return( NO);
}


- (BOOL) immutableDataMethodsAreFast
{
   return( YES);
}


//
// The next routines should be overridden by subclasses, if they can
// handle it.
//
- (void) encryptData:(NSMutableData *) data
             withKey:(MulleSymmetricCipherKey *) key
{
   unsigned int  n;
   NSData        *values;

#if DEBUG
   NSAssert( [data isKindOfClass:[NSData class]], @"loss1");
   NSAssert( [key isKindOfClass:[MulleSymmetricCipherKey class]], @"loss2");
#endif   
   values = nil;
   if( n = [self sizeOfInitialValues])
      values = [NSMutableData dataWithLength:n];
   [self encryptData:data
             withKey:key
   withInitialValues:values];
}


- (void) decryptData:(NSMutableData *) data
             withKey:(MulleSymmetricCipherKey *) key
{
   unsigned int  n;
   NSData        *values;

#if DEBUG
   NSAssert( [data isKindOfClass:[NSData class]], @"loss1");
   NSAssert( [key isKindOfClass:[MulleSymmetricCipherKey class]], @"loss2");
#endif   
   values = nil;
   if( n = [self sizeOfInitialValues])
      values = [NSMutableData dataWithLength:n];
   [self decryptData:data
             withKey:key
   withInitialValues:values];
}


- (NSData *) encryptedData:(NSData *) data
                   withKey:(MulleSymmetricCipherKey *) key
{
   unsigned int  n;
   NSData        *values;

#if DEBUG
   NSAssert( [data isKindOfClass:[NSData class]], @"loss1");
   NSAssert( [key isKindOfClass:[MulleSymmetricCipherKey class]], @"loss2");
#endif   
   values = nil;
   if( n = [self sizeOfInitialValues])
      values = [NSMutableData dataWithLength:n];
   return( [self encryptedData:data
                       withKey:key
             withInitialValues:values]);
}


- (NSData *) decryptedData:(NSData *) data
                   withKey:(MulleSymmetricCipherKey *) key;

{
   unsigned int  n;
   NSData        *values;

#if DEBUG
   NSAssert( [data isKindOfClass:[NSData class]], @"loss1");
   NSAssert( [key isKindOfClass:[MulleSymmetricCipherKey class]], @"loss2");
#endif   
   values = nil;
   if( n = [self sizeOfInitialValues])
      values = [NSMutableData dataWithLength:n];
   return( [self decryptedData:data
                       withKey:key
             withInitialValues:values]);
}


//
// we have defined that non mutable conversion is the default, so
//
- (void) encryptData:(NSMutableData *) data
             withKey:(MulleSymmetricCipherKey *) key
   withInitialValues:(NSData *) values
{
   NSData        *code;
   int           diff;
   unsigned int  len;
   
#if DEBUG
   NSAssert( [data isKindOfClass:[NSMutableData class]], @"loss1");
   NSAssert( [key isKindOfClass:[MulleSymmetricCipherKey class]], @"loss2");
   NSAssert( ! values || [values isKindOfClass:[NSData class]], @"loss3");
#endif   
   code = [self encryptedData:data
                      withKey:key
            withInitialValues:values];
   len  = [code length];
   diff = (long) len - (long) [data length];
   if( diff > 0)
      [data increaseLengthBy:diff];

   // a simple memcpy is probably better...
   [data replaceBytesInRange:NSMakeRange( 0, len)
                   withBytes:[code bytes]];
}


- (void) decryptData:(NSMutableData *) data
             withKey:(MulleSymmetricCipherKey *) key
   withInitialValues:(NSData *) values
{
   NSData        *plain;
   int           diff;
   unsigned int  len;

#if DEBUG
   NSAssert( [data isKindOfClass:[NSMutableData class]], @"loss1");
   NSAssert( [key isKindOfClass:[MulleSymmetricCipherKey class]], @"loss2");
   NSAssert( ! values || [values isKindOfClass:[NSData class]], @"loss3");
#endif   
   plain = [self decryptedData:data
                       withKey:key
             withInitialValues:values];
   len  = [plain length];
   diff = (long) len - (long) [data length];
   if( diff > 0)
      [data increaseLengthBy:diff];

   // a simple memcpy is probably better...
   [data replaceBytesInRange:NSMakeRange( 0, len)
                   withBytes:[plain bytes]];
}




@end
